@@ -27,6 +27,8 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include <glib.h>
 #include <glib-object.h>
@@ -49,6 +51,10 @@ struct _GdmDisplayAccessFilePrivate
 #define GDM_DISPLAY_ACCESS_COOKIE_SIZE 16
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 static void gdm_display_access_file_finalize (GObject * object);
 
 enum
@@ -220,23 +226,42 @@ _create_xauth_file_for_user (const char  *username,
                              GError     **error)
 {
         char   *template;
-        GError *open_error;
+        char   *dir_name;
+        char   *auth_filename;
         int     fd;
         FILE   *fp;
         uid_t   uid;
         gid_t   gid;
 
+        g_assert (filename != NULL);
+
+        *filename = NULL;
+
+        template = NULL;
+        auth_filename = NULL;
         fp = NULL;
+        fd = -1;
 
-        template = g_strdup_printf (".gdm-xauth-%s.XXXXXX", username);
+        /* Create directory if not exist, then set permission 01775 and ownership root:gdm */
+        if (g_file_test (GDM_XAUTH_DIR, G_FILE_TEST_IS_DIR) == FALSE) {
+                g_unlink (GDM_XAUTH_DIR);
+                if (g_mkdir (GDM_XAUTH_DIR, S_ISVTX | S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0) {
+                        g_set_error (error,
+                                     G_FILE_ERROR,
+                                     g_file_error_from_errno (errno),
+                                     "%s", g_strerror (errno));
+                        goto out;
+                }
 
-        open_error = NULL;
-        fd = g_file_open_tmp (template, filename, &open_error);
-        g_free (template);
-
-        if (fd < 0) {
-                g_propagate_error (error, open_error);
-                goto out;
+                g_chmod (GDM_XAUTH_DIR, S_ISVTX | S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
+                _get_uid_and_gid_for_user ("gdm", &uid, &gid);
+                if (chown (GDM_XAUTH_DIR, 0, gid) != 0) {
+                        g_warning ("Unable to change owner of '%s'",
+                                   GDM_XAUTH_DIR);
+                }
+        } else {
+                /* if it does exist make sure it has correct mode 01775 */
+                g_chmod (GDM_XAUTH_DIR, S_ISVTX | S_IRWXU |S_IRWXG | S_IROTH | S_IXOTH);
         }
 
         if (!_get_uid_and_gid_for_user (username, &uid, &gid)) {
@@ -245,24 +270,80 @@ _create_xauth_file_for_user (const char  *username,
                              GDM_DISPLAY_ERROR_GETTING_USER_INFO,
                              _("could not find user \"%s\" on system"),
                              username);
-                close (fd);
-                fd = -1;
                 goto out;
 
         }
 
+        template = g_strdup_printf (GDM_XAUTH_DIR
+                                    "/auth-for-%s-XXXXXX",
+                                    username);
+
+        g_debug ("GdmDisplayAccessFile: creating xauth directory %s", template);
+        /* Initially create with mode 01700 then later chmod after we create database */
+        errno = 0;
+        dir_name = gdm_make_temp_dir (template);
+        if (dir_name == NULL) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             "Unable to create temp dir from tempalte '%s': %s",
+                             template,
+                             g_strerror (errno));
+                goto out;
+        }
+
+        g_debug ("GdmDisplayAccessFile: chowning %s to %u:%u",
+                 dir_name, (guint)uid, (guint)gid);
+        errno = 0;
+        if (chown (dir_name, uid, gid) < 0) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             "Unable to change permission of '%s': %s",
+                             dir_name,
+                             g_strerror (errno));
+                goto out;
+        }
+
+        auth_filename = g_build_filename (dir_name, "database", NULL);
+
+        g_debug ("GdmDisplayAccessFile: creating %s", auth_filename);
+        /* mode 00600 */
+        errno = 0;
+        fd = g_open (auth_filename,
+                     O_RDWR | O_CREAT | O_EXCL | O_BINARY,
+                     S_IRUSR | S_IWUSR);
+
+        if (fd < 0) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             "Unable to open '%s': %s",
+                             auth_filename,
+                             g_strerror (errno));
+                goto out;
+        }
+
+        g_debug ("GdmDisplayAccessFile: chowning %s to %u:%u", auth_filename, (guint)uid, (guint)gid);
+        errno = 0;
         if (fchown (fd, uid, gid) < 0) {
                 g_set_error (error,
                              G_FILE_ERROR,
                              g_file_error_from_errno (errno),
-                             "%s", g_strerror (errno));
+                             "Unable to change owner for '%s': %s",
+                             auth_filename,
+                             g_strerror (errno));
                 close (fd);
                 fd = -1;
                 goto out;
         }
 
-        fp = fdopen (fd, "w");
+        /* now open up permissions on per-session directory */
+        g_debug ("GdmDisplayAccessFile: chmoding %s to 1777", dir_name);
+        g_chmod (dir_name, S_ISVTX | S_IRWXU | S_IRWXG | S_IRWXO);
 
+        errno = 0;
+        fp = fdopen (fd, "w");
         if (fp == NULL) {
                 g_set_error (error,
                              G_FILE_ERROR,
@@ -272,7 +353,19 @@ _create_xauth_file_for_user (const char  *username,
                 fd = -1;
                 goto out;
         }
+
+        *filename = auth_filename;
+        auth_filename = NULL;
+
+        /* don't close it */
+        fd = -1;
 out:
+        g_free (template);
+        g_free (auth_filename);
+        if (fd != -1) {
+                close (fd);
+        }
+
         return fp;
 }
 
@@ -339,7 +432,7 @@ gdm_display_access_file_add_display (GdmDisplayAccessFile  *file,
                                      gsize                 *cookie_size,
                                      GError               **error)
 {
-        GError *add_error;
+        GError  *add_error;
         gboolean display_added;
 
         g_return_val_if_fail (file != NULL, FALSE);
@@ -481,17 +574,37 @@ gdm_display_access_file_remove_display (GdmDisplayAccessFile  *file,
 void
 gdm_display_access_file_close (GdmDisplayAccessFile  *file)
 {
+        char *auth_dir;
+
         g_return_if_fail (file != NULL);
         g_return_if_fail (file->priv->fp != NULL);
         g_return_if_fail (file->priv->path != NULL);
 
-        g_unlink (file->priv->path);
-        if (file->priv->path != NULL) {
-                g_free (file->priv->path);
-                file->priv->path = NULL;
-                g_object_notify (G_OBJECT (file), "path");
+        errno = 0;
+        if (g_unlink (file->priv->path) != 0) {
+                g_warning ("GdmDisplayAccessFile: Unable to remove X11 authority database '%s': %s",
+                           file->priv->path,
+                           g_strerror (errno));
         }
 
+        /* still try to remove dir even if file remove failed,
+           may have already been removed by someone else */
+        /* we own the parent directory too */
+        auth_dir = g_path_get_dirname (file->priv->path);
+        if (auth_dir != NULL) {
+                errno = 0;
+                if (g_rmdir (auth_dir) != 0) {
+                        g_warning ("GdmDisplayAccessFile: Unable to remove X11 authority directory '%s': %s",
+                                   auth_dir,
+                                   g_strerror (errno));
+                }
+                g_free (auth_dir);
+        }
+
+        g_free (file->priv->path);
+        file->priv->path = NULL;
+        g_object_notify (G_OBJECT (file), "path");
+
         fclose (file->priv->fp);
         file->priv->fp = NULL;
 }
