@@ -415,6 +415,14 @@ on_greeter_language_selected (GdmGreeterServer *greeter_server,
         gdm_session_select_language (GDM_SESSION (slave->priv->session), text);
 }
 
+static void
+on_greeter_layout_selected (GdmGreeterServer *greeter_server,
+                            const char       *text,
+                            GdmFactorySlave  *slave)
+{
+        gdm_session_select_layout (GDM_SESSION (slave->priv->session), text);
+}
+
 static void
 on_greeter_user_selected (GdmGreeterServer *greeter_server,
                           const char       *text,
@@ -481,7 +489,7 @@ run_greeter (GdmFactorySlave *slave)
         /* FIXME: send a signal back to the master */
 
         /* Run the init script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init/Default", "gdm");
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", "gdm");
 
         slave->priv->greeter_server = gdm_greeter_server_new (display_id);
         g_signal_connect (slave->priv->greeter_server,
@@ -504,6 +512,10 @@ run_greeter (GdmFactorySlave *slave)
                           "language-selected",
                           G_CALLBACK (on_greeter_language_selected),
                           slave);
+        g_signal_connect (slave->priv->greeter_server,
+                          "layout-selected",
+                          G_CALLBACK (on_greeter_layout_selected),
+                          slave);
         g_signal_connect (slave->priv->greeter_server,
                           "user-selected",
                           G_CALLBACK (on_greeter_user_selected),
