@@ -18,8 +18,8 @@
  * 02111-1307, USA.
  */
 
-#ifndef __GDM_SESSION_H
-#define __GDM_SESSION_H
+#ifndef __GDM_SESSION_WORKER_H
+#define __GDM_SESSION_WORKER_H
 
 #include <glib-object.h>
 
