@@ -62,6 +62,7 @@
 
 #include "gdm-settings-client.h"
 #include "gdm-settings-keys.h"
+#include "gdm-profile.h"
 
 #include "gdm-greeter-login-window.h"
 #include "gdm-user-chooser-widget.h"
@@ -121,12 +122,15 @@ struct GdmGreeterLoginWindowPrivate
         gboolean         banner_message_enabled;
         guint            gconf_cnxn;
 
+        guint            dialog_mode;
+
         gboolean         timed_login_enabled;
         guint            timed_login_delay;
         char            *timed_login_username;
         guint            timed_login_timeout_id;
 
         guint            animation_timeout_id;
+        guint            sensitize_power_buttons_timeout_id;
 
         guint            login_button_handler_id;
         guint            start_session_handler_id;
@@ -156,6 +160,10 @@ static void     gdm_greeter_login_window_class_init   (GdmGreeterLoginWindowClas
 static void     gdm_greeter_login_window_init         (GdmGreeterLoginWindow      *greeter_login_window);
 static void     gdm_greeter_login_window_finalize     (GObject                    *object);
 
+static void     restart_timed_login_timeout (GdmGreeterLoginWindow *login_window);
+static void     on_user_unchosen            (GdmUserChooserWidget *user_chooser,
+                                             GdmGreeterLoginWindow *login_window);
+
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
 static void
@@ -206,11 +214,7 @@ set_busy (GdmGreeterLoginWindow *login_window)
 static void
 set_ready (GdmGreeterLoginWindow *login_window)
 {
-        GdkCursor *cursor;
-
-        cursor = gdk_cursor_new (GDK_LEFT_PTR);
-        gdk_window_set_cursor (GTK_WIDGET (login_window)->window, cursor);
-        gdk_cursor_unref (cursor);
+        gdk_window_set_cursor (GTK_WIDGET (login_window)->window, NULL);
 }
 
 static void
@@ -224,6 +228,8 @@ set_sensitive (GdmGreeterLoginWindow *login_window,
 
         box = glade_xml_get_widget (login_window->priv->xml, "buttonbox");
         gtk_widget_set_sensitive (box, sensitive);
+
+        gtk_widget_set_sensitive (login_window->priv->user_chooser, sensitive);
 }
 
 static void
@@ -250,6 +256,46 @@ set_message (GdmGreeterLoginWindow *login_window,
         gtk_label_set_text (GTK_LABEL (label), text);
 }
 
+static void
+on_user_interaction (GdmGreeterLoginWindow *login_window)
+{
+        g_debug ("GdmGreeterLoginWindow: user is interacting with session!\n");
+        restart_timed_login_timeout (login_window);
+}
+
+static GdkFilterReturn
+on_xevent (XEvent                *xevent,
+           GdkEvent              *event,
+           GdmGreeterLoginWindow *login_window)
+{
+        switch (xevent->xany.type) {
+                case KeyPress:
+                case KeyRelease:
+                case ButtonPress:
+                case ButtonRelease:
+                        on_user_interaction (login_window);
+                        break;
+                case  PropertyNotify:
+                        if (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("_NET_WM_USER_TIME")) {
+                                on_user_interaction (login_window);
+                        }
+                        break;
+
+                default:
+                        break;
+        }
+
+        return GDK_FILTER_CONTINUE;
+}
+
+static void
+stop_watching_for_user_interaction (GdmGreeterLoginWindow *login_window)
+{
+        gdk_window_remove_filter (NULL,
+                                  (GdkFilterFunc) on_xevent,
+                                  login_window);
+}
+
 static void
 remove_timed_login_timeout (GdmGreeterLoginWindow *login_window)
 {
@@ -258,13 +304,14 @@ remove_timed_login_timeout (GdmGreeterLoginWindow *login_window)
                 g_source_remove (login_window->priv->timed_login_timeout_id);
                 login_window->priv->timed_login_timeout_id = 0;
         }
+
+        stop_watching_for_user_interaction (login_window);
 }
 
 static void
 _gdm_greeter_login_window_set_interactive (GdmGreeterLoginWindow *login_window,
                                            gboolean               is_interactive)
 {
-
         if (login_window->priv->is_interactive != is_interactive) {
                 login_window->priv->is_interactive = is_interactive;
                 g_object_notify (G_OBJECT (login_window), "is-interactive");
@@ -284,6 +331,14 @@ timed_login_timer (GdmGreeterLoginWindow *login_window)
         return FALSE;
 }
 
+static void
+watch_for_user_interaction (GdmGreeterLoginWindow *login_window)
+{
+        gdk_window_add_filter (NULL,
+                               (GdkFilterFunc) on_xevent,
+                               login_window);
+}
+
 static void
 restart_timed_login_timeout (GdmGreeterLoginWindow *login_window)
 {
@@ -291,7 +346,7 @@ restart_timed_login_timeout (GdmGreeterLoginWindow *login_window)
 
         if (login_window->priv->timed_login_enabled) {
                 g_debug ("GdmGreeterLoginWindow: adding timed login timer");
-
+                watch_for_user_interaction (login_window);
                 login_window->priv->timed_login_timeout_id = g_timeout_add_seconds (login_window->priv->timed_login_delay,
                                                                                     (GSourceFunc)timed_login_timer,
                                                                                     login_window);
@@ -319,6 +374,19 @@ show_widget (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static void
+sensitize_widget (GdmGreeterLoginWindow *login_window,
+                  const char            *name,
+                  gboolean               sense)
+{
+        GtkWidget *widget;
+
+        widget = glade_xml_get_widget (login_window->priv->xml, name);
+        if (widget != NULL) {
+                gtk_widget_set_sensitive (widget, sense);
+        }
+}
+
 static gboolean
 get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
 {
@@ -410,6 +478,84 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static gboolean
+can_suspend (GdmGreeterLoginWindow *login_window)
+{
+        DBusGConnection *connection;
+        DBusGProxy      *proxy;
+        GError          *error;
+        gboolean         ret;
+        gboolean         res;
+
+        error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+        if (error != NULL) {
+                g_warning ("Couldn't connect to power manager: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+        proxy = dbus_g_proxy_new_for_name (connection,
+                                           GPM_DBUS_NAME,
+                                           GPM_DBUS_PATH,
+                                           GPM_DBUS_INTERFACE);
+        ret = FALSE;
+
+        res = dbus_g_proxy_call (proxy, "CanSuspend",
+                                 &error,
+                                 G_TYPE_INVALID,
+                                 G_TYPE_BOOLEAN,
+                                 &ret,
+                                 G_TYPE_INVALID);
+        if (! res) {
+                if (error != NULL) {
+                        g_warning ("Could not ask power manager if user can suspend: %s",
+                                   error->message);
+                        g_error_free (error);
+                }
+                ret = FALSE;
+        }
+
+        return ret;
+}
+
+static void
+remove_sensitize_power_buttons_timeout (GdmGreeterLoginWindow *login_window)
+{
+        if (login_window->priv->sensitize_power_buttons_timeout_id > 0) {
+                g_source_remove (login_window->priv->sensitize_power_buttons_timeout_id);
+                login_window->priv->sensitize_power_buttons_timeout_id = 0;
+        }
+}
+
+static gboolean
+sensitize_power_buttons_timeout (GdmGreeterLoginWindow *login_window)
+{
+        switch (login_window->priv->dialog_mode) {
+        case MODE_SELECTION:
+                sensitize_widget (login_window, "shutdown-button", TRUE);
+                sensitize_widget (login_window, "restart-button", TRUE);
+                sensitize_widget (login_window, "suspend-button", TRUE);
+                sensitize_widget (login_window, "disconnect-button", TRUE);
+                break;
+        case MODE_AUTHENTICATION:
+                break;
+        default:
+                g_assert_not_reached ();
+        }
+
+        login_window->priv->sensitize_power_buttons_timeout_id = 0;
+        return FALSE;
+}
+
+static void
+add_sensitize_power_buttons_timeout (GdmGreeterLoginWindow *login_window)
+{
+        remove_sensitize_power_buttons_timeout (login_window);
+        login_window->priv->sensitize_power_buttons_timeout_id = g_timeout_add_seconds (1,
+                                                                                   (GSourceFunc)sensitize_power_buttons_timeout,
+                                                                                   login_window);
+}
+
 static void
 switch_mode (GdmGreeterLoginWindow *login_window,
              int                    number)
@@ -418,29 +564,50 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         GtkWidget  *user_chooser;
         GtkWidget  *box;
         gboolean    show_restart_buttons;
-
-        /* FIXME: do animation */
-        default_name = NULL;
+        gboolean    show_suspend_button;
 
         show_restart_buttons = get_show_restart_buttons (login_window);
+        show_suspend_button = can_suspend (login_window);
+
+        /* we want to run this even if we're supposed to
+           be in the mode already so that we reset everything
+           to a known state */
+        login_window->priv->dialog_mode = number;
+
+        default_name = NULL;
+
+        remove_sensitize_power_buttons_timeout (login_window);
 
         switch (number) {
         case MODE_SELECTION:
+                gtk_widget_set_size_request (GTK_WIDGET (login_window), -1, -1);
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
 
                 show_widget (login_window, "cancel-button", FALSE);
+
                 show_widget (login_window, "shutdown-button",
                              login_window->priv->display_is_local && show_restart_buttons);
                 show_widget (login_window, "restart-button",
                              login_window->priv->display_is_local && show_restart_buttons);
                 show_widget (login_window, "suspend-button",
-                             login_window->priv->display_is_local && show_restart_buttons);
+                             login_window->priv->display_is_local && show_restart_buttons && show_suspend_button);
                 show_widget (login_window, "disconnect-button",
                              ! login_window->priv->display_is_local);
+
                 show_widget (login_window, "auth-input-box", FALSE);
+
+                add_sensitize_power_buttons_timeout (login_window);
+                sensitize_widget (login_window, "shutdown-button", FALSE);
+                sensitize_widget (login_window, "restart-button", FALSE);
+                sensitize_widget (login_window, "suspend-button", FALSE);
+                sensitize_widget (login_window, "disconnect-button", FALSE);
+
                 default_name = NULL;
                 break;
         case MODE_AUTHENTICATION:
+                gtk_widget_set_size_request (GTK_WIDGET (login_window),
+                                             GTK_WIDGET (login_window)->allocation.width,
+                                             -1);
                 show_widget (login_window, "cancel-button", TRUE);
                 show_widget (login_window, "shutdown-button", FALSE);
                 show_widget (login_window, "restart-button", FALSE);
@@ -525,31 +692,6 @@ do_suspend (GdmGreeterLoginWindow *login_window)
         g_object_unref (proxy);
 }
 
-static void
-do_cancel (GdmGreeterLoginWindow *login_window)
-{
-        gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
-                                           GDM_USER_CHOOSER_USER_AUTO, 0);
-        login_window->priv->timed_login_enabled = FALSE;
-        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-
-        if (login_window->priv->start_session_handler_id > 0) {
-                g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
-                login_window->priv->start_session_handler_id = 0;
-        }
-
-        switch_mode (login_window, MODE_SELECTION);
-        set_busy (login_window);
-        set_sensitive (login_window, FALSE);
-        set_message (login_window, "");
-        remove_timed_login_timeout (login_window);
-        _gdm_greeter_login_window_set_interactive (login_window, FALSE);
-
-        g_signal_emit (login_window, signals[CANCELLED], 0);
-
-        set_ready (login_window);
-}
-
 static void
 delete_entry_text (GtkWidget *entry)
 {
@@ -570,6 +712,27 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         GtkWidget  *label;
 
         g_debug ("GdmGreeterLoginWindow: Resetting dialog");
+        set_busy (login_window);
+        set_sensitive (login_window, FALSE);
+
+        if (login_window->priv->timed_login_enabled) {
+                gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                   GDM_USER_CHOOSER_USER_AUTO, 0);
+                remove_timed_login_timeout (login_window);
+                login_window->priv->timed_login_enabled = FALSE;
+        }
+        _gdm_greeter_login_window_set_interactive (login_window, FALSE);
+
+        g_signal_handlers_block_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                         G_CALLBACK (on_user_unchosen), login_window);
+        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
+        g_signal_handlers_unblock_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                           G_CALLBACK (on_user_unchosen), login_window);
+
+        if (login_window->priv->start_session_handler_id > 0) {
+                g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
+                login_window->priv->start_session_handler_id = 0;
+        }
 
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
 
@@ -581,15 +744,22 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), "");
 
-        set_message (login_window, "");
-
-        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-
         switch_mode (login_window, MODE_SELECTION);
 
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
+        gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+}
+
+static void
+do_cancel (GdmGreeterLoginWindow *login_window)
+{
+        /* need to wait for response from backend */
+        set_message (login_window, _("Cancelling..."));
+        set_busy (login_window);
+        set_sensitive (login_window, FALSE);
+        g_signal_emit (login_window, signals[CANCELLED], 0);
 }
 
 gboolean
@@ -597,6 +767,8 @@ gdm_greeter_login_window_ready (GdmGreeterLoginWindow *login_window)
 {
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
+        reset_dialog (login_window);
+
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -636,6 +808,7 @@ gdm_greeter_login_window_problem (GdmGreeterLoginWindow *login_window,
         g_debug ("GdmGreeterLoginWindow: problem: %s", text);
 
         set_message (GDM_GREETER_LOGIN_WINDOW (login_window), text);
+        gdk_window_beep (GTK_WIDGET (login_window)->window);
 
         return TRUE;
 }
@@ -645,16 +818,30 @@ gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_windo
                                               const char            *username,
                                               int                    delay)
 {
+        static gboolean timed_login_already_enabled;
+
         g_return_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window));
 
         g_debug ("GdmGreeterLoginWindow: requested automatic login for user '%s' in %d seconds", username, delay);
 
+        if (login_window->priv->timed_login_username != NULL) {
+                timed_login_already_enabled = TRUE;
+                g_free (login_window->priv->timed_login_username);
+        } else {
+                timed_login_already_enabled = FALSE;
+        }
         login_window->priv->timed_login_username = g_strdup (username);
         login_window->priv->timed_login_delay = delay;
 
-        reset_dialog (login_window);
+        if (login_window->priv->dialog_mode != MODE_SELECTION) {
+                reset_dialog (login_window);
+        }
         gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
-        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), GDM_USER_CHOOSER_USER_AUTO);
+
+        if (!timed_login_already_enabled) {
+                gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                              GDM_USER_CHOOSER_USER_AUTO);
+        }
 }
 
 static void
@@ -718,6 +905,8 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
 
+        gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+
         return TRUE;
 }
 
@@ -743,6 +932,8 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
 
+        gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+
         return TRUE;
 }
 
@@ -955,20 +1146,42 @@ static PolKitAction *
 get_action_from_error (GError *error)
 {
         PolKitAction *action;
-        const char   *paction;
+        char         *paction;
+        char         *p;
 
         action = polkit_action_new ();
 
         paction = NULL;
         if (g_str_has_prefix (error->message, "Not privileged for action: ")) {
-                paction = error->message + strlen ("Not privileged for action: ");
+                paction = g_strdup (error->message + strlen ("Not privileged for action: "));
+                p = strchr (paction, ' ');
+                if (p != NULL) {
+                        *p = '\0';
+                }
         }
         g_debug ("GdmGreeterLoginWindow: Requesting priv for '%s'", paction);
 
         polkit_action_set_action_id (action, paction);
 
+        g_free (paction);
+
         return action;
 }
+
+static PolKitResult
+get_result_from_error (GError *error)
+{
+        PolKitResult result = POLKIT_RESULT_UNKNOWN;
+        const char  *p;
+
+        p = strrchr (error->message, ' ');
+        if (p != NULL) {
+                p++;
+                polkit_result_from_string_representation (p, &result);
+        }
+
+        return result;
+}
 #endif
 
 static void
@@ -995,11 +1208,40 @@ do_system_restart (GdmGreeterLoginWindow *login_window)
 
                 if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
                         PolKitAction *action;
+                        PolKitAction *action2;
+                        PolKitResult  result;
+                        GtkWidget    *dialog;
                         guint         xid;
                         pid_t         pid;
 
+                        result = get_result_from_error (error);
                         action = get_action_from_error (error);
 
+                        if (result == POLKIT_RESULT_NO) {
+                                action2 = polkit_action_new ();
+                                polkit_action_set_action_id (action2,
+                                                             "org.freedesktop.consolekit.system.restart-multiple-users");
+                                dialog = gtk_message_dialog_new (GTK_WINDOW (login_window),
+                                                                 GTK_DIALOG_MODAL,
+                                                                 GTK_MESSAGE_ERROR,
+                                                                 GTK_BUTTONS_OK,
+                                                                 _("Failed to restart computer"));
+                                if (polkit_action_equal (action, action2)) {
+                                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                                                  _("You are not allowed to restart the computer "
+                                                                                    "because multiple users are logged in"));
+                                }
+                                gtk_dialog_run (GTK_DIALOG (dialog));
+                                gtk_widget_destroy (dialog);
+
+                                polkit_action_unref (action);
+                                polkit_action_unref (action2);
+
+                                g_error_free (error);
+
+                                return;
+                        }
+
                         xid = 0;
                         pid = getpid ();
 
@@ -1047,14 +1289,41 @@ do_system_stop (GdmGreeterLoginWindow *login_window)
 
                 if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
                         PolKitAction *action;
+                        PolKitAction *action2;
+                        PolKitResult  result;
+                        GtkWidget    *dialog;
                         guint         xid;
                         pid_t         pid;
 
                         xid = 0;
                         pid = getpid ();
 
+                        result = get_result_from_error (error);
                         action = get_action_from_error (error);
 
+                        if (result == POLKIT_RESULT_NO) {
+                                action2 = polkit_action_new ();
+                                polkit_action_set_action_id (action2,
+                                                             "org.freedesktop.consolekit.system.stop-multiple-users");
+                                dialog = gtk_message_dialog_new (GTK_WINDOW (login_window),
+                                                                 GTK_DIALOG_MODAL,
+                                                                 GTK_MESSAGE_ERROR,
+                                                                 GTK_BUTTONS_OK,
+                                                                 _("Failed to stop computer"));
+                                if (polkit_action_equal (action, action2)) {
+                                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                                                  _("You are not allowed to stop the computer "
+                                                                                    "because multiple users are logged in"));
+                                }
+                                gtk_dialog_run (GTK_DIALOG (dialog));
+                                gtk_widget_destroy (dialog);
+
+                                polkit_action_unref (action);
+                                polkit_action_unref (action2);
+
+                                return;
+                        }
+
                         g_error_free (error);
                         error = NULL;
                         res = polkit_gnome_auth_obtain (action,
@@ -1240,6 +1509,8 @@ create_computer_info (GdmGreeterLoginWindow *login_window)
 {
         GtkWidget *label;
 
+        gdm_profile_start (NULL);
+
         label = glade_xml_get_widget (login_window->priv->xml, "computer-info-name-label");
         if (label != NULL) {
                 gtk_label_set_text (GTK_LABEL (label), g_get_host_name ());
@@ -1252,6 +1523,8 @@ create_computer_info (GdmGreeterLoginWindow *login_window)
                 gtk_label_set_text (GTK_LABEL (label), version);
                 g_free (version);
         }
+
+        gdm_profile_end (NULL);
 }
 
 #define INVISIBLE_CHAR_DEFAULT       '*'
@@ -1276,12 +1549,16 @@ custom_widget_constructor (GladeXML              *xml,
         g_assert (name != NULL);
         g_assert (GDM_IS_GREETER_LOGIN_WINDOW (login_window));
 
+        gdm_profile_start (NULL);
+
         widget = NULL;
 
         if (strcmp (name, "user-chooser") == 0) {
                widget = gdm_user_chooser_widget_new ();
         }
 
+        gdm_profile_end (NULL);
+
         return widget;
 }
 
@@ -1293,6 +1570,8 @@ load_theme (GdmGreeterLoginWindow *login_window)
         GtkWidget *box;
         GtkWidget *image;
 
+        gdm_profile_start (NULL);
+
         glade_set_custom_handler ((GladeXMLCustomWidgetHandler) custom_widget_constructor,
                                   login_window);
         login_window->priv->xml = glade_xml_new (GLADEDIR "/" GLADE_XML_FILE,
@@ -1379,53 +1658,8 @@ load_theme (GdmGreeterLoginWindow *login_window)
         g_signal_connect (box, "button-press-event", G_CALLBACK (on_computer_info_label_button_press), login_window);
 
         switch_mode (login_window, MODE_SELECTION);
-}
 
-static gboolean
-fit_window_to_children (GdmGreeterLoginWindow *window)
-{
-        int x;
-        int y;
-        int width;
-        int height;
-        int height_step;
-        int width_step;
-
-        /* FIXME: this animation logic is really dumb
-         */
-
-        if (!GTK_WIDGET_REALIZED (GTK_WIDGET (window))) {
-                return FALSE;
-        }
-
-        gdk_window_get_geometry (GTK_WIDGET (window)->window,
-                                 &x, &y, &width, &height, NULL);
-
-        if (height == GTK_WIDGET (window)->requisition.height) {
-                return FALSE;
-        }
-
-        if (width < GTK_WIDGET (window)->requisition.width) {
-                width_step = MIN (1, GTK_WIDGET (window)->requisition.width - width);
-        } else if (width > GTK_WIDGET (window)->requisition.width) {
-                width_step = -1 * MIN (1, width - GTK_WIDGET (window)->requisition.width);
-        } else {
-                width_step = 0;
-        }
-
-        if (height < GTK_WIDGET (window)->requisition.height) {
-                height_step = MIN ((int) 25, GTK_WIDGET (window)->requisition.height - height);
-        } else if (height > GTK_WIDGET (window)->requisition.height) {
-                height_step = -1 * MIN ((int) 25, height - GTK_WIDGET (window)->requisition.height);
-        } else {
-                height_step = 0;
-        }
-
-        gdk_window_resize (GTK_WIDGET (window)->window,
-                           width + width_step,
-                           height + height_step);
-
-        return TRUE;
+        gdm_profile_end (NULL);
 }
 
 static gboolean
@@ -1438,7 +1672,9 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
         login_window = GDM_GREETER_LOGIN_WINDOW (widget);
 
         if (event->keyval == GDK_Escape) {
-                reset_dialog (GDM_GREETER_LOGIN_WINDOW (widget));
+                if (login_window->priv->dialog_mode == MODE_AUTHENTICATION) {
+                        do_cancel (GDM_GREETER_LOGIN_WINDOW (widget));
+                }
         }
 
         capslock_on = is_capslock_on ();
@@ -1465,41 +1701,19 @@ gdm_greeter_login_window_size_request (GtkWidget      *widget,
         screen_w = gdk_screen_get_width (gtk_widget_get_screen (widget));
         screen_h = gdk_screen_get_height (gtk_widget_get_screen (widget));
 
-        gtk_widget_size_request (GTK_BIN (widget)->child, &child_requisition);
+        gtk_widget_get_child_requisition (GTK_BIN (widget)->child, &child_requisition);
         *requisition = child_requisition;
 
         requisition->width += 2 * GTK_CONTAINER (widget)->border_width;
         requisition->height += 2 * GTK_CONTAINER (widget)->border_width;
 
-        requisition->width = MIN (requisition->width, .50 * screen_w);
+        /* Make width be at least 33% screen width
+         * and height be at most 80% of screen height
+         */
+        requisition->width = MAX (requisition->width, .33 * screen_w);
         requisition->height = MIN (requisition->height, .80 * screen_h);
 }
 
-static void
-clear_animation_timeout_id (GdmGreeterLoginWindow *window)
-{
-        window->priv->animation_timeout_id = 0;
-}
-
-static void
-gdm_greeter_login_window_size_allocate (GtkWidget      *widget,
-                                        GtkAllocation  *allocation)
-{
-        GdmGreeterLoginWindow *window;
-
-        window = GDM_GREETER_LOGIN_WINDOW (widget);
-
-        if (window->priv->animation_timeout_id == 0) {
-                window->priv->animation_timeout_id =
-                    g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 20,
-                                        (GSourceFunc) fit_window_to_children,
-                                        widget,
-                                        (GDestroyNotify) clear_animation_timeout_id);
-        }
-
-        GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_allocate (widget, allocation);
-}
-
 static void
 update_banner_message (GdmGreeterLoginWindow *login_window)
 {
@@ -1548,6 +1762,8 @@ gdm_greeter_login_window_constructor (GType                  type,
 {
         GdmGreeterLoginWindow      *login_window;
 
+        gdm_profile_start (NULL);
+
         login_window = GDM_GREETER_LOGIN_WINDOW (G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->constructor (type,
                                                                                                                       n_construct_properties,
                                                                                                                       construct_properties));
@@ -1556,6 +1772,8 @@ gdm_greeter_login_window_constructor (GType                  type,
         load_theme (login_window);
         update_banner_message (login_window);
 
+        gdm_profile_end (NULL);
+
         return G_OBJECT (login_window);
 }
 
@@ -1572,7 +1790,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
 
         widget_class->key_press_event = gdm_greeter_login_window_key_press_event;
         widget_class->size_request = gdm_greeter_login_window_size_request;
-        widget_class->size_allocate = gdm_greeter_login_window_size_allocate;
 
         signals [BEGIN_AUTO_LOGIN] =
                 g_signal_new ("begin-auto-login",
@@ -1705,23 +1922,46 @@ on_gconf_key_changed (GConfClient           *client,
         }
 }
 
+static gboolean
+on_window_state_event (GtkWidget           *widget,
+                       GdkEventWindowState *event,
+                       gpointer             data)
+{
+        if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) {
+                g_debug ("GdmGreeterLoginWindow: window iconified");
+                gtk_window_deiconify (GTK_WINDOW (widget));
+        }
+
+        return FALSE;
+}
+
 static void
 gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
 {
+        gdm_profile_start (NULL);
+
         login_window->priv = GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE (login_window);
 
         login_window->priv->timed_login_enabled = FALSE;
 
+        login_window->priv->dialog_mode = MODE_SELECTION;
+
         gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
-        gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);
+        /*gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);*/
         gtk_window_set_position (GTK_WINDOW (login_window), GTK_WIN_POS_CENTER_ALWAYS);
         gtk_window_set_deletable (GTK_WINDOW (login_window), FALSE);
         gtk_window_set_decorated (GTK_WINDOW (login_window), FALSE);
+        gtk_window_set_keep_below (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
         gtk_container_set_border_width (GTK_CONTAINER (login_window), 25);
 
+        g_signal_connect (login_window,
+                          "window-state-event",
+                          G_CALLBACK (on_window_state_event),
+                          NULL);
+
         login_window->priv->client = gconf_client_get_default ();
         gconf_client_add_dir (login_window->priv->client,
                               KEY_GREETER_DIR,
@@ -1733,6 +1973,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
                                                                   login_window,
                                                                   NULL,
                                                                   NULL);
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -1751,6 +1992,8 @@ gdm_greeter_login_window_finalize (GObject *object)
                 g_object_unref (login_window->priv->client);
         }
 
+        remove_sensitize_power_buttons_timeout (login_window);
+
         G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->finalize (object);
 }
 
@@ -1761,6 +2004,7 @@ gdm_greeter_login_window_new (gboolean is_local)
 
         object = g_object_new (GDM_TYPE_GREETER_LOGIN_WINDOW,
                                "display-is-local", is_local,
+                               "resizable", FALSE,
                                NULL);
 
         return GTK_WIDGET (object);
