@@ -57,19 +57,14 @@ typedef struct _GdmLocale {
         char *territory_code;
         char *codeset;
         char *modifier;
-        char *title;
-        char *language;
-        char *territory;
 } GdmLocale;
 
 static GHashTable *gdm_languages_map;
 static GHashTable *gdm_territories_map;
 static GHashTable *gdm_available_locales_map;
 
-static gboolean gdm_languages_is_initialized = FALSE;
-
 static void
-chooser_locale_free (GdmLocale *locale)
+gdm_locale_free (GdmLocale *locale)
 {
         if (locale == NULL) {
                 return;
@@ -77,9 +72,6 @@ chooser_locale_free (GdmLocale *locale)
 
         g_free (locale->id);
         g_free (locale->name);
-        g_free (locale->title);
-        g_free (locale->language);
-        g_free (locale->territory);
         g_free (locale->codeset);
         g_free (locale->modifier);
         g_free (locale);
@@ -135,7 +127,7 @@ gdm_parse_language_name (const char *name,
                           "(@(?P<modifier>[[:ascii:]]+))?$",
                           0, 0, &error);
         if (re == NULL) {
-                g_critical (error->message);
+                g_critical ("%s", error->message);
                 return;
         }
 
@@ -253,81 +245,6 @@ gdm_normalize_language_name (const char *name)
         return normalized_name;
 }
 
-static char *
-utf8_convert (const char *str,
-              int         len)
-{
-        char *utf8;
-
-        utf8 = g_locale_to_utf8 (str, len, NULL, NULL, NULL);
-
-        /* if we couldn't convert text from locale then
-         * assume utf-8 and hope for the best */
-        if (utf8 == NULL) {
-                char *p;
-                char *q;
-
-                if (len < 0) {
-                        utf8 = g_strdup (str);
-                } else {
-                        utf8 = g_strndup (str, len);
-                }
-
-                p = utf8;
-                while (*p != '\0' && !g_utf8_validate ((const char *)p, -1, (const char **)&q)) {
-                        *q = '?';
-                        p = q + 1;
-                }
-        }
-
-        return utf8;
-}
-
-/* Magic number at the beginning of a locale data file for CATEGORY.  */
-#define LIMAGIC(category) \
-  (category == LC_COLLATE                                               \
-   ? ((unsigned int) (0x20051014 ^ (category)))                         \
-   : ((unsigned int) (0x20031115 ^ (category))))
-
-
-/* This seems to be specified by ISO/IEC 14652 */
-static void
-get_lc_identification (GdmLocale *locale,
-                       void             *data,
-                       gsize             size)
-{
-        struct {
-                unsigned int magic;
-                unsigned int nstrings;
-                unsigned int strindex[];
-        } *filedata = data;
-
-#ifdef LC_IDENTIFICATION
-        if (filedata->magic == LIMAGIC (LC_IDENTIFICATION)
-            && (sizeof *filedata + (filedata->nstrings * sizeof (unsigned int)) <= size)) {
-
-#define GET_HANDLE(idx) ((char *) data + filedata->strindex[_NL_ITEM_INDEX (_NL_IDENTIFICATION_##idx)])
-
-                locale->title = utf8_convert (GET_HANDLE (TITLE), -1);
-                locale->language = utf8_convert (GET_HANDLE (LANGUAGE), -1);
-                locale->territory = utf8_convert (GET_HANDLE (TERRITORY), -1);
-        }
-#endif
-}
-
-struct nameent
-{
-        char    *name;
-        uint32_t locrec_offset;
-};
-
-static int
-nameentcmp (const void *a, const void *b)
-{
-        return strcoll (((const struct nameent *) a)->name,
-                        ((const struct nameent *) b)->name);
-}
-
 static gboolean
 language_name_is_valid (const char *language_name)
 {
@@ -367,7 +284,107 @@ language_name_is_utf8 (const char *language_name)
         return is_utf8;
 }
 
-static void
+static gboolean
+language_name_has_translations (const char *language_name)
+{
+        GDir        *dir;
+        char        *path;
+        const char  *name;
+        gboolean     has_translations;
+
+        path = g_build_filename (GNOMELOCALEDIR, language_name, "LC_MESSAGES", NULL);
+
+        has_translations = FALSE;
+        dir = g_dir_open (path, 0, NULL);
+        g_free (path);
+
+        if (dir == NULL) {
+                goto out;
+        }
+
+        do {
+                name = g_dir_read_name (dir);
+
+                if (name == NULL) {
+                        break;
+                }
+
+                if (g_str_has_suffix (name, ".mo")) {
+                        has_translations = TRUE;
+                        break;
+                }
+        } while (name != NULL);
+        g_dir_close (dir);
+
+out:
+        return has_translations;
+}
+
+static gboolean
+add_locale (const char *language_name)
+{
+        GdmLocale *locale;
+        GdmLocale *old_locale;
+        char      *name;
+
+        if (language_name_is_utf8 (language_name)) {
+                name = g_strdup (language_name);
+        } else {
+                name = g_strdup_printf ("%s.utf8", language_name);
+
+                if (!language_name_is_utf8 (name)) {
+                        g_free (name);
+                        return FALSE;
+                }
+        }
+
+        if (!language_name_is_valid (name)) {
+                g_free (name);
+                return FALSE;
+        }
+
+
+        locale = g_new0 (GdmLocale, 1);
+        gdm_parse_language_name (name,
+                                 &locale->language_code,
+                                 &locale->territory_code,
+                                 &locale->codeset,
+                                 &locale->modifier);
+        g_free (name);
+        name = NULL;
+
+        locale->id = construct_language_name (locale->language_code, locale->territory_code,
+                                              NULL, locale->modifier);
+        locale->name = construct_language_name (locale->language_code, locale->territory_code,
+                                                locale->codeset, locale->modifier);
+
+        if (!language_name_has_translations (locale->name) &&
+            !language_name_has_translations (locale->id) &&
+            !language_name_has_translations (locale->language_code)) {
+                gdm_locale_free (locale);
+                return FALSE;
+        }
+
+        old_locale = g_hash_table_lookup (gdm_available_locales_map, locale->id);
+        if (old_locale != NULL) {
+                if (strlen (old_locale->name) > strlen (locale->name)) {
+                        gdm_locale_free (locale);
+                        return FALSE;
+                }
+        }
+
+        g_hash_table_insert (gdm_available_locales_map, g_strdup (locale->id), locale);
+
+        return TRUE;
+}
+
+struct nameent
+{
+        char    *name;
+        uint32_t locrec_offset;
+};
+
+static gboolean
 collect_locales_from_archive (void)
 {
         GMappedFile        *mapped;
@@ -379,15 +396,18 @@ collect_locales_from_archive (void)
         uint32_t            used;
         uint32_t            cnt;
         gsize               len;
+        gboolean            locales_collected;
 
         error = NULL;
         mapped = g_mapped_file_new (ARCHIVE_FILE, FALSE, &error);
         if (mapped == NULL) {
                 g_warning ("Mapping failed for %s: %s", ARCHIVE_FILE, error->message);
                 g_error_free (error);
-                return;
+                return FALSE;
         }
 
+        locales_collected = FALSE;
+
         addr = g_mapped_file_get_contents (mapped);
         len = g_mapped_file_get_length (mapped);
 
@@ -409,57 +429,17 @@ collect_locales_from_archive (void)
                 }
         }
 
-        /* Sort the names.  */
-        qsort (names, used, sizeof (struct nameent), nameentcmp);
-
         for (cnt = 0; cnt < used; ++cnt) {
-                struct locrecent *locrec;
-                GdmLocale        *locale;
-                GdmLocale        *old_locale;
-
-                if (!language_name_is_valid (names[cnt].name) ||
-                    !language_name_is_utf8 (names[cnt].name)) {
-                        continue;
-                }
-
-                locale = g_new0 (GdmLocale, 1);
-
-                gdm_parse_language_name (names[cnt].name,
-                                         &locale->language_code,
-                                         &locale->territory_code,
-                                         &locale->codeset,
-                                         &locale->modifier);
-
-                locale->id = construct_language_name (locale->language_code, locale->territory_code,
-                                                      NULL, locale->modifier);
-                locale->name = construct_language_name (locale->language_code, locale->territory_code,
-                                                        locale->codeset, locale->modifier);
-
-                old_locale = g_hash_table_lookup (gdm_available_locales_map, locale->id);
-                if (old_locale != NULL) {
-                        if (strlen (old_locale->name) > strlen (locale->name)) {
-                                    chooser_locale_free (locale);
-                                    continue;
-                        }
-                }
-
-
-                locrec = (struct locrecent *) (addr + names[cnt].locrec_offset);
-
-#ifdef LC_IDENTIFICATION
-                get_lc_identification (locale,
-                                       addr + locrec->record[LC_IDENTIFICATION].offset,
-                                       locrec->record[LC_IDENTIFICATION].len);
-#endif
-
-                g_hash_table_insert (gdm_available_locales_map, g_strdup (locale->id), locale);
+                add_locale (names[cnt].name);
         }
 
         g_free (names);
 
+        locales_collected = TRUE;
  out:
 
         g_mapped_file_free (mapped);
+        return locales_collected;
 }
 
 static int
@@ -479,7 +459,7 @@ select_dirs (const struct dirent *dirent)
                                 struct stat st;
                                 char       *path;
 
-                                path = g_build_filename (LIBLOCALEDIR, dirent->d_name, NULL);
+                                path = g_build_filename (GNOMELOCALEDIR, dirent->d_name, NULL);
                                 if (g_stat (path, &st) == 0) {
                                         mode = st.st_mode;
                                 }
@@ -499,61 +479,10 @@ collect_locales_from_directory (void)
         int             ndirents;
         int             cnt;
 
-        ndirents = scandir (LIBLOCALEDIR, &dirents, select_dirs, alphasort);
+        ndirents = scandir (GNOMELOCALEDIR, &dirents, select_dirs, alphasort);
 
         for (cnt = 0; cnt < ndirents; ++cnt) {
-                char      *path;
-                GdmLocale *locale;
-                GdmLocale *old_locale;
-                gboolean   res;
-
-                if (!language_name_is_valid (dirents[cnt]->d_name) ||
-                    !language_name_is_utf8 (dirents[cnt]->d_name)) {
-                        continue;
-                }
-
-                locale = g_new0 (GdmLocale, 1);
-                gdm_parse_language_name (dirents[cnt]->d_name,
-                                         &locale->language_code,
-                                         &locale->territory_code,
-                                         &locale->codeset,
-                                         &locale->modifier);
-
-                locale->id = construct_language_name (locale->language_code, locale->territory_code,
-                                                      NULL, locale->modifier);
-                locale->name = construct_language_name (locale->language_code, locale->territory_code,
-                                                        locale->codeset, locale->modifier);
-
-                old_locale = g_hash_table_lookup (gdm_available_locales_map, locale->id);
-                if (old_locale != NULL) {
-                        if (strlen (old_locale->name) > strlen (locale->name)) {
-                                chooser_locale_free (locale);
-                                continue;
-                        }
-                }
-
-                /* try to get additional information from LC_IDENTIFICATION */
-                path = g_build_filename (LIBLOCALEDIR, dirents[cnt]->d_name, "LC_IDENTIFICATION", NULL);
-                res = g_file_test (path, G_FILE_TEST_IS_REGULAR);
-                if (res) {
-                        GMappedFile      *mapped;
-                        GError           *error;
-
-                        error = NULL;
-                        mapped = g_mapped_file_new (path, FALSE, &error);
-                        if (mapped == NULL) {
-                                g_warning ("Mapping failed for %s: %s", path, error->message);
-                                g_error_free (error);
-                        } else {
-                                get_lc_identification (locale,
-                                                       g_mapped_file_get_contents (mapped),
-                                                       g_mapped_file_get_length (mapped));
-                                g_mapped_file_free (mapped);
-                        }
-                }
-                g_free (path);
-
-                g_hash_table_insert (gdm_available_locales_map, g_strdup (locale->id), locale);
+                add_locale (dirents[cnt]->d_name);
         }
 
         if (ndirents > 0) {
@@ -561,23 +490,38 @@ collect_locales_from_directory (void)
         }
 }
 
-static void
-collect_locales_from_aliases (void)
-{
-        /* FIXME: */
-}
-
 static void
 collect_locales (void)
 {
 
         if (gdm_available_locales_map == NULL) {
-                gdm_available_locales_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) chooser_locale_free);
+                gdm_available_locales_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) gdm_locale_free);
         }
 
-        collect_locales_from_archive ();
-        collect_locales_from_directory ();
-        collect_locales_from_aliases ();
+        if (collect_locales_from_archive ()) {
+                return;
+        } else {
+                g_warning ("Could not read list of available locales from libc, "
+                           "guessing possible locales from available translations, "
+                           "but list may be incomplete!");
+
+                collect_locales_from_directory ();
+        }
+}
+
+static gboolean
+is_fallback_language (const char *code)
+{
+        const char *fallback_language_names[] = { "C", "POSIX", NULL };
+        int i;
+
+        for (i = 0; fallback_language_names[i] != NULL; i++) {
+                if (strcmp (code, fallback_language_names[i]) == 0) {
+                        return TRUE;
+                }
+        }
+
+        return FALSE;
 }
 
 static const char *
@@ -588,6 +532,10 @@ get_language (const char *code)
 
         g_assert (code != NULL);
 
+        if (is_fallback_language (code)) {
+                return "Unspecified";
+        }
+
         len = strlen (code);
         if (len != 2 && len != 3) {
                 return NULL;
@@ -630,16 +578,23 @@ get_translated_language (const char *code,
                 const char  *translated_name;
                 char        *old_locale;
 
-                old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-                setlocale (LC_MESSAGES, locale);
-                translated_name = dgettext ("iso_639", language);
+                if (locale != NULL) {
+                        old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+                        setlocale (LC_MESSAGES, locale);
+                }
 
-                name = get_first_item_in_semicolon_list (translated_name);
+                if (is_fallback_language (code)) {
+                        name = _("Unspecified");
+                } else {
+                        translated_name = dgettext ("iso_639", language);
+                        name = get_first_item_in_semicolon_list (translated_name);
+                }
 
-                setlocale (LC_MESSAGES, old_locale);
-                g_free (old_locale);
-
-            }
+                if (locale != NULL) {
+                        setlocale (LC_MESSAGES, old_locale);
+                        g_free (old_locale);
+                }
+        }
 
         return name;
 }
@@ -676,13 +631,18 @@ get_translated_territory (const char *code,
                 const char *translated_territory;
                 char       *old_locale;
 
-                old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-                setlocale (LC_MESSAGES, locale);
-                translated_territory = dgettext ("iso_3166", territory);
-                setlocale (LC_MESSAGES, old_locale);
-                g_free (old_locale);
+                if (locale != NULL) {
+                        old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+                        setlocale (LC_MESSAGES, locale);
+                }
 
+                translated_territory = dgettext ("iso_3166", territory);
                 name = get_first_item_in_semicolon_list (translated_territory);
+
+                if (locale != NULL) {
+                        setlocale (LC_MESSAGES, old_locale);
+                        g_free (old_locale);
+                }
         }
 
         return name;
@@ -929,7 +889,8 @@ territories_init (void)
 }
 
 char *
-gdm_get_language_from_name (const char *name)
+gdm_get_language_from_name (const char *name,
+                            const char *locale)
 {
         char *full_language;
         char *language_code;
@@ -937,12 +898,12 @@ gdm_get_language_from_name (const char *name)
         const char *language;
         const char *territory;
 
-        if (!gdm_languages_is_initialized) {
-                collect_locales ();
+        if (gdm_languages_map == NULL) {
                 languages_init ();
-                territories_init ();
+        }
 
-                gdm_languages_is_initialized = TRUE;
+        if (gdm_territories_map == NULL) {
+                territories_init ();
         }
 
         language_code = NULL;
@@ -956,10 +917,10 @@ gdm_get_language_from_name (const char *name)
                 goto out;
         }
 
-        language = get_translated_language (language_code, name);
+        language = get_translated_language (language_code, locale);
 
         if (territory_code != NULL) {
-                territory = get_translated_territory (territory_code, name);
+                territory = get_translated_territory (territory_code, locale);
         } else {
                 territory = NULL;
         }
@@ -986,12 +947,8 @@ gdm_get_all_language_names (void)
         gpointer key, value;
         GPtrArray *array;
 
-        if (!gdm_languages_is_initialized) {
+        if (gdm_available_locales_map == NULL) {
                 collect_locales ();
-                languages_init ();
-                territories_init ();
-
-                gdm_languages_is_initialized = TRUE;
         }
 
         array = g_ptr_array_new ();
