@@ -62,16 +62,39 @@ gdm_user_chooser_dialog_get_chosen_user_name (GdmUserChooserDialog *dialog)
         return user_name;
 }
 
+void
+gdm_user_chooser_dialog_set_show_other_user (GdmUserChooserDialog *dialog,
+                                             gboolean              show_user)
+{
+        g_return_if_fail (GDM_IS_USER_CHOOSER_DIALOG (dialog));
+
+        gdm_user_chooser_widget_set_show_other_user (GDM_USER_CHOOSER_WIDGET (dialog->priv->chooser_widget), show_user);
+}
+
+void
+gdm_user_chooser_dialog_set_show_guest_user (GdmUserChooserDialog *dialog,
+                                             gboolean              show_user)
+{
+        g_return_if_fail (GDM_IS_USER_CHOOSER_DIALOG (dialog));
+
+        gdm_user_chooser_widget_set_show_guest_user (GDM_USER_CHOOSER_WIDGET (dialog->priv->chooser_widget), show_user);
+}
+
+void
+gdm_user_chooser_dialog_set_show_auto_user (GdmUserChooserDialog *dialog,
+                                            gboolean              show_user)
+{
+        g_return_if_fail (GDM_IS_USER_CHOOSER_DIALOG (dialog));
+
+        gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (dialog->priv->chooser_widget), show_user);
+}
+
 static void
 gdm_user_chooser_dialog_set_property (GObject        *object,
                                       guint           prop_id,
                                       const GValue   *value,
                                       GParamSpec     *pspec)
 {
-        GdmUserChooserDialog *self;
-
-        self = GDM_USER_CHOOSER_DIALOG (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -85,10 +108,6 @@ gdm_user_chooser_dialog_get_property (GObject        *object,
                                       GValue         *value,
                                       GParamSpec     *pspec)
 {
-        GdmUserChooserDialog *self;
-
-        self = GDM_USER_CHOOSER_DIALOG (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -102,9 +121,6 @@ gdm_user_chooser_dialog_constructor (GType                  type,
                                      GObjectConstructParam *construct_properties)
 {
         GdmUserChooserDialog      *user_chooser_dialog;
-        GdmUserChooserDialogClass *klass;
-
-        klass = GDM_USER_CHOOSER_DIALOG_CLASS (g_type_class_peek (GDM_TYPE_USER_CHOOSER_DIALOG));
 
         user_chooser_dialog = GDM_USER_CHOOSER_DIALOG (G_OBJECT_CLASS (gdm_user_chooser_dialog_parent_class)->constructor (type,
                                                                                                                            n_construct_properties,
@@ -116,10 +132,6 @@ gdm_user_chooser_dialog_constructor (GType                  type,
 static void
 gdm_user_chooser_dialog_dispose (GObject *object)
 {
-        GdmUserChooserDialog *user_chooser_dialog;
-
-        user_chooser_dialog = GDM_USER_CHOOSER_DIALOG (object);
-
         G_OBJECT_CLASS (gdm_user_chooser_dialog_parent_class)->dispose (object);
 }
 
@@ -154,8 +166,7 @@ gdm_user_chooser_dialog_init (GdmUserChooserDialog *dialog)
         dialog->priv = GDM_USER_CHOOSER_DIALOG_GET_PRIVATE (dialog);
 
         dialog->priv->chooser_widget = gdm_user_chooser_widget_new ();
-        gdm_user_chooser_widget_set_show_other_user (GDM_USER_CHOOSER_WIDGET (dialog->priv->chooser_widget), TRUE);
-        gdm_user_chooser_widget_set_show_guest_user (GDM_USER_CHOOSER_WIDGET (dialog->priv->chooser_widget), TRUE);
+
         gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), dialog->priv->chooser_widget);
 
         gtk_dialog_add_buttons (GTK_DIALOG (dialog),
