@@ -34,9 +34,14 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
+#include <gconf/gconf-client.h>
+
 #include "gdm-user-manager.h"
 #include "gdm-user-chooser-widget.h"
 
+
+#define KEY_DISABLE_USER_LIST "/apps/gdm/simple-greeter/disable_user_list"
+
 enum {
         USER_NO_DISPLAY              = 1 << 0,
         USER_ACCOUNT_DISABLED        = 1 << 1,
@@ -46,7 +51,7 @@ enum {
 
 #define GDM_USER_CHOOSER_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_USER_CHOOSER_WIDGET, GdmUserChooserWidgetPrivate))
 
-#define ICON_SIZE 64
+#define ICON_SIZE 96
 
 struct GdmUserChooserWidgetPrivate
 {
@@ -56,9 +61,11 @@ struct GdmUserChooserWidgetPrivate
         GdkPixbuf      *logged_in_pixbuf;
         GdkPixbuf      *stock_person_pixbuf;
 
+        guint           loaded : 1;
         guint           show_other_user : 1;
         guint           show_guest_user : 1;
         guint           show_auto_user : 1;
+        guint           show_normal_users : 1;
 };
 
 enum {
@@ -76,7 +83,7 @@ add_user_other (GdmUserChooserWidget *widget)
 {
         gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
                                      GDM_USER_CHOOSER_USER_OTHER,
-                                     widget->priv->stock_person_pixbuf,
+                                     NULL,
                                      _("Other..."),
                                      _("Choose a different account"),
                                      0,
@@ -102,7 +109,7 @@ add_user_auto (GdmUserChooserWidget *widget)
 {
         gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
                                      GDM_USER_CHOOSER_USER_AUTO,
-                                     widget->priv->stock_person_pixbuf,
+                                     NULL,
                                      _("Automatic Login"),
                                      _("Automatically login to the system after selecting options"),
                                      0,
@@ -231,6 +238,25 @@ gdm_user_chooser_widget_get_property (GObject        *object,
         }
 }
 
+static gboolean
+is_user_list_disabled (GdmUserChooserWidget *widget)
+{
+        GConfClient *client;
+        GError      *error;
+        gboolean     result;
+
+        client = gconf_client_get_default ();
+        error = NULL;
+        result = gconf_client_get_bool (client, KEY_DISABLE_USER_LIST, &error);
+        if (error != NULL) {
+                g_debug ("GdmUserChooserWidget: unable to get disable-user-list configuration: %s", error->message);
+                g_error_free (error);
+        }
+        g_object_unref (client);
+
+        return result;
+}
+
 static GObject *
 gdm_user_chooser_widget_constructor (GType                  type,
                                      guint                  n_construct_properties,
@@ -247,6 +273,8 @@ gdm_user_chooser_widget_constructor (GType                  type,
         gdm_user_chooser_widget_set_show_auto_user (user_chooser_widget, FALSE);
         gdm_user_chooser_widget_set_show_other_user (user_chooser_widget, TRUE);
 
+        user_chooser_widget->priv->show_normal_users = !is_user_list_disabled (user_chooser_widget);
+
         return G_OBJECT (user_chooser_widget);
 }
 
@@ -387,14 +415,17 @@ setup_icons (GdmUserChooserWidget *widget)
 }
 
 static void
-on_user_added (GdmUserManager       *manager,
-               GdmUser              *user,
-               GdmUserChooserWidget *widget)
+add_user (GdmUserChooserWidget *widget,
+          GdmUser              *user)
 {
         GdkPixbuf    *pixbuf;
         char         *tooltip;
         gboolean      is_logged_in;
 
+        if (!widget->priv->show_normal_users) {
+                return;
+        }
+
         pixbuf = gdm_user_render_icon (user, ICON_SIZE);
         if (pixbuf == NULL && widget->priv->stock_person_pixbuf != NULL) {
                 pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
@@ -425,6 +456,38 @@ on_user_added (GdmUserManager       *manager,
         }
 }
 
+static void
+on_users_loaded (GdmUserManager       *manager,
+                 GdmUserChooserWidget *widget)
+{
+        GSList *users;
+
+        widget->priv->loaded = TRUE;
+
+        g_debug ("GdmUserChooserWidget: Users loaded");
+
+        users = gdm_user_manager_list_users (manager);
+        while (users != NULL) {
+                add_user (widget, users->data);
+                users = g_slist_delete_link (users, users);
+        }
+
+        gdm_chooser_widget_activate_on_one_item (GDM_CHOOSER_WIDGET (widget),
+                                                 TRUE);
+}
+
+static void
+on_user_added (GdmUserManager       *manager,
+               GdmUser              *user,
+               GdmUserChooserWidget *widget)
+{
+        /* wait for all users to be loaded */
+        if (! widget->priv->loaded) {
+                return;
+        }
+        add_user (widget, user);
+}
+
 static void
 on_user_removed (GdmUserManager       *manager,
                  GdmUser              *user,
@@ -433,6 +496,10 @@ on_user_removed (GdmUserManager       *manager,
         const char *user_name;
 
         g_debug ("GdmUserChooserWidget: User removed: %s", gdm_user_get_user_name (user));
+        /* wait for all users to be loaded */
+        if (! widget->priv->loaded) {
+                return;
+        }
 
         user_name = gdm_user_get_user_name (user);
 
@@ -495,6 +562,10 @@ gdm_user_chooser_widget_init (GdmUserChooserWidget *widget)
                           "user-removed",
                           G_CALLBACK (on_user_removed),
                           widget);
+        g_signal_connect (widget->priv->manager,
+                          "users-loaded",
+                          G_CALLBACK (on_users_loaded),
+                          widget);
         g_signal_connect (widget->priv->manager,
                           "user-is-logged-in-changed",
                           G_CALLBACK (on_user_is_logged_in_changed),
@@ -528,8 +599,6 @@ gdm_user_chooser_widget_new (void)
         GObject *object;
 
         object = g_object_new (GDM_TYPE_USER_CHOOSER_WIDGET,
-                               "inactive-text", _("_Users:"),
-                               "active-text", _("_User:"),
                                NULL);
 
         return GTK_WIDGET (object);
