@@ -774,7 +774,7 @@ get_uid_from_session_id (GdmUserManager *manager,
 {
         DBusGProxy      *proxy;
         GError          *error;
-        int              uid;
+        guint            uid;
         gboolean         res;
 
         proxy = dbus_g_proxy_new_for_name (manager->priv->connection,
@@ -791,7 +791,7 @@ get_uid_from_session_id (GdmUserManager *manager,
                                  "GetUnixUser",
                                  &error,
                                  G_TYPE_INVALID,
-                                 G_TYPE_INT, &uid,
+                                 G_TYPE_UINT, &uid,
                                  G_TYPE_INVALID);
         g_object_unref (proxy);
 
@@ -940,11 +940,11 @@ get_seat_proxy (GdmUserManager *manager)
 
         dbus_g_proxy_add_signal (proxy,
                                  "SessionAdded",
-                                 G_TYPE_STRING,
+                                 DBUS_TYPE_G_OBJECT_PATH,
                                  G_TYPE_INVALID);
         dbus_g_proxy_add_signal (proxy,
                                  "SessionRemoved",
-                                 G_TYPE_STRING,
+                                 DBUS_TYPE_G_OBJECT_PATH,
                                  G_TYPE_INVALID);
         dbus_g_proxy_connect_signal (proxy,
                                      "SessionAdded",
@@ -1084,7 +1084,7 @@ parse_ck_history_line (const char *line,
         error = NULL;
         re = g_regex_new ("(?P<username>[0-9a-zA-Z]+)[ ]+(?P<frequency>[0-9]+)", 0, 0, &error);
         if (re == NULL) {
-                g_critical (error->message);
+                g_critical ("%s", error->message);
                 goto out;
         }
 
@@ -1188,6 +1188,8 @@ ck_history_watch (GIOChannel     *source,
         }
 
         if (done) {
+                g_signal_emit (G_OBJECT (manager), signals[USERS_LOADED], 0);
+
                 manager->priv->ck_history_id = 0;
                 return FALSE;
         }
@@ -1378,7 +1380,6 @@ reload_users_timeout (GdmUserManager *manager)
         reload_users (manager);
         manager->priv->reload_id = 0;
 
-        g_signal_emit (G_OBJECT (manager), signals[USERS_LOADED], 0);
         return FALSE;
 }
 
