@@ -33,12 +33,15 @@
 
 #include "gdm-log.h"
 #include "gdm-common.h"
+#include "gdm-signal-handler.h"
 #include "gdm-settings-client.h"
 #include "gdm-settings-keys.h"
+#include "gdm-profile.h"
 
 #include "gdm-greeter-session.h"
 
 #define ACCESSIBILITY_KEY         "/desktop/gnome/interface/accessibility"
+#define DEBUG_KEY                 "/apps/gdm/simple-greeter/debug"
 
 static Atom AT_SPI_IOR;
 
@@ -50,6 +53,11 @@ assistive_registry_launch (void)
         const char *command;
         char      **argv;
         gboolean    res;
+        gboolean    ret;
+
+        ret = FALSE;
+
+        gdm_profile_start (NULL);
 
         command = AT_SPI_REGISTRYD_DIR "/at-spi-registryd";
 
@@ -58,7 +66,7 @@ assistive_registry_launch (void)
         res = g_shell_parse_argv (command, NULL, &argv, &error);
         if (! res) {
                 g_warning ("Unable to parse command: %s", error->message);
-                return FALSE;
+                goto out;
         }
 
         error = NULL;
@@ -76,27 +84,32 @@ assistive_registry_launch (void)
 
         if (! res) {
                 g_warning ("Unable to run command %s: %s", command, error->message);
-                return FALSE;
+                goto out;
         }
 
         if (kill (pid, 0) < 0) {
                 g_warning ("at-spi-registryd not running");
-                return FALSE;
+                goto out;
         }
 
-        return TRUE;
+        ret = TRUE;
+ out:
+        gdm_profile_end (NULL);
+
+        return ret;
 }
 
 static GdkFilterReturn
 filter_watch (GdkXEvent *xevent,
               GdkEvent  *event,
-              gpointer   data)
+              GMainLoop *loop)
 {
         XEvent *xev = (XEvent *)xevent;
 
         if (xev->xany.type == PropertyNotify
             && xev->xproperty.atom == AT_SPI_IOR) {
-                gtk_main_quit ();
+                g_debug ("a11y registry started");
+                g_main_loop_quit (loop);
 
                 return GDK_FILTER_REMOVE;
         }
@@ -105,11 +118,11 @@ filter_watch (GdkXEvent *xevent,
 }
 
 static gboolean
-filter_timeout (gpointer data)
+filter_timeout (GMainLoop *loop)
 {
         g_warning ("The accessibility registry was not found.");
 
-        gtk_main_quit ();
+        g_main_loop_quit (loop);
 
         return FALSE;
 }
@@ -119,6 +132,11 @@ assistive_registry_start (void)
 {
         GdkWindow *root;
         guint      tid;
+        GMainLoop *loop;
+
+        gdm_profile_start (NULL);
+
+        g_debug ("Starting a11y registry");
 
         root = gdk_get_default_root_window ();
 
@@ -126,20 +144,25 @@ assistive_registry_start (void)
                 AT_SPI_IOR = XInternAtom (GDK_DISPLAY (), "AT_SPI_IOR", False);
         }
 
-        gdk_window_set_events (root,  GDK_PROPERTY_CHANGE_MASK);
+        gdk_window_set_events (root, GDK_PROPERTY_CHANGE_MASK);
 
         if ( ! assistive_registry_launch ()) {
                 g_warning ("The accessibility registry could not be started.");
                 return;
         }
 
-        gdk_window_add_filter (root, filter_watch, NULL);
-        tid = g_timeout_add_seconds (5, filter_timeout, NULL);
+        loop = g_main_loop_new (NULL, FALSE);
+        gdk_window_add_filter (root, (GdkFilterFunc)filter_watch, loop);
+        tid = g_timeout_add_seconds (5, (GSourceFunc)filter_timeout, loop);
 
-        gtk_main ();
+        g_main_loop_run (loop);
 
-        gdk_window_remove_filter (root, filter_watch, NULL);
+        gdk_window_remove_filter (root, (GdkFilterFunc)filter_watch, loop);
         g_source_remove (tid);
+
+        g_main_loop_unref (loop);
+
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -153,6 +176,8 @@ at_set_gtk_modules (void)
         gboolean    found_atk_bridge;
         int         n;
 
+        gdm_profile_start (NULL);
+
         n = 0;
         modules_list = NULL;
         found_gail = FALSE;
@@ -161,9 +186,9 @@ at_set_gtk_modules (void)
         if ((old = g_getenv ("GTK_MODULES")) != NULL) {
                 modules = g_strsplit (old, ":", -1);
                 for (n = 0; modules[n]; n++) {
-                        if (!strcmp (modules[n], "gail")) {
+                        if (strcmp (modules[n], "gail") == 0) {
                                 found_gail = TRUE;
-                        } else if (!strcmp (modules[n], "atk-bridge")) {
+                        } else if (strcmp (modules[n], "atk-bridge") == 0) {
                                 found_atk_bridge = TRUE;
                         }
 
@@ -192,6 +217,8 @@ at_set_gtk_modules (void)
         g_setenv ("GTK_MODULES", g_strjoinv (":", modules), TRUE);
         g_strfreev (modules);
         g_slist_free (modules_list);
+
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -201,13 +228,23 @@ load_a11y (void)
         gboolean           a_t_support;
         GConfClient       *gconf_client;
 
+        gdm_profile_start (NULL);
+
         gconf_client = gconf_client_get_default ();
 
         env_a_t_support = g_getenv ("GNOME_ACCESSIBILITY");
         if (env_a_t_support) {
                 a_t_support = atoi (env_a_t_support);
         } else {
-                a_t_support = gconf_client_get_bool (gconf_client, ACCESSIBILITY_KEY, NULL);
+                GConfValue *val;
+
+                a_t_support = TRUE;
+
+                val = gconf_client_get_without_default (gconf_client, ACCESSIBILITY_KEY, NULL);
+                if (val != NULL) {
+                        a_t_support = gconf_value_get_bool (val);
+                        gconf_value_free (val);
+                }
         }
 
         if (a_t_support) {
@@ -216,6 +253,84 @@ load_a11y (void)
         }
 
         g_object_unref (gconf_client);
+
+        gdm_profile_end (NULL);
+}
+
+static gboolean
+is_debug_set (void)
+{
+        GConfClient *client;
+        gboolean     is;
+
+        /* enable debugging for unstable builds */
+        if (gdm_is_version_unstable ()) {
+                return TRUE;
+        }
+
+        client = gconf_client_get_default ();
+        is = gconf_client_get_bool (client, DEBUG_KEY, NULL);
+        g_object_unref (client);
+
+        return is;
+}
+
+
+static gboolean
+signal_cb (int      signo,
+           gpointer data)
+{
+        int ret;
+
+        g_debug ("Got callback for signal %d", signo);
+
+        ret = TRUE;
+
+        switch (signo) {
+        case SIGFPE:
+        case SIGPIPE:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down abnormally.", signo);
+                ret = FALSE;
+
+                break;
+
+        case SIGINT:
+        case SIGTERM:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down normally.", signo);
+                ret = FALSE;
+
+                break;
+
+        case SIGHUP:
+                g_debug ("Got HUP signal");
+                /* FIXME:
+                 * Reread config stuff like system config files, VPN service files, etc
+                 */
+                ret = TRUE;
+
+                break;
+
+        case SIGUSR1:
+                g_debug ("Got USR1 signal");
+                /* FIXME:
+                 * Play with log levels or something
+                 */
+                ret = TRUE;
+
+                gdm_log_toggle_debug ();
+
+                break;
+
+        default:
+                g_debug ("Caught unhandled signal %d", signo);
+                ret = TRUE;
+
+                break;
+        }
+
+        return ret;
 }
 
 int
@@ -224,6 +339,7 @@ main (int argc, char *argv[])
         GError            *error;
         GdmGreeterSession *session;
         gboolean           res;
+        GdmSignalHandler  *signal_handler;
 
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -235,9 +351,12 @@ main (int argc, char *argv[])
 
         g_type_init ();
 
+        gdm_profile_start ("Initializing settings client");
         if (! gdm_settings_client_init (GDMCONFDIR "/gdm.schemas", "/")) {
+                g_critical ("Unable to initialize settings client");
                 exit (1);
         }
+        gdm_profile_end ("Initializing settings client");
 
         g_debug ("Greeter session pid=%d display=%s xauthority=%s",
                  (int)getpid (),
@@ -248,7 +367,7 @@ main (int argc, char *argv[])
         /*sleep (15);*/
 
         gdm_log_init ();
-        gdm_log_set_debug (TRUE);
+        gdm_log_set_debug (is_debug_set ());
 
         gdk_init (&argc, &argv);
 
@@ -256,11 +375,21 @@ main (int argc, char *argv[])
 
         gtk_init (&argc, &argv);
 
+        signal_handler = gdm_signal_handler_new ();
+        gdm_signal_handler_add_fatal (signal_handler);
+        gdm_signal_handler_add (signal_handler, SIGTERM, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGFPE, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
+
+        gdm_profile_start ("Creating new greeter session");
         session = gdm_greeter_session_new ();
         if (session == NULL) {
                 g_critical ("Unable to create greeter session");
                 exit (1);
         }
+        gdm_profile_end ("Creating new greeter session");
 
         error = NULL;
         res = gdm_greeter_session_start (session, &error);
